@@ -459,6 +459,7 @@ struct connman_device *connman_inet_create_device(int index)
 
 	if (__connman_element_device_isfiltered(devname) == TRUE) {
 		connman_info("Ignoring interface %s (filtered)", devname);
+		g_free(devname);
 		return NULL;
 	}
 
@@ -732,7 +733,8 @@ int connman_inet_del_network_route(int index, const char *host)
 	return err;
 }
 
-int connman_inet_del_ipv6_host_route(int index, const char *host)
+int connman_inet_del_ipv6_network_route(int index, const char *host,
+						unsigned char prefix_len)
 {
 	struct in6_rtmsg rt;
 	int sk, err;
@@ -744,7 +746,7 @@ int connman_inet_del_ipv6_host_route(int index, const char *host)
 
 	memset(&rt, 0, sizeof(rt));
 
-	rt.rtmsg_dst_len = 128;
+	rt.rtmsg_dst_len = prefix_len;
 
 	err = inet_pton(AF_INET6, host, &rt.rtmsg_dst);
 	if (err < 0)
@@ -770,8 +772,14 @@ out:
 	return err;
 }
 
-int connman_inet_add_ipv6_host_route(int index, const char *host,
-						const char *gateway)
+int connman_inet_del_ipv6_host_route(int index, const char *host)
+{
+	return connman_inet_del_ipv6_network_route(index, host, 128);
+}
+
+int connman_inet_add_ipv6_network_route(int index, const char *host,
+					const char *gateway,
+						unsigned char prefix_len)
 {
 	struct in6_rtmsg rt;
 	int sk, err;
@@ -783,7 +791,7 @@ int connman_inet_add_ipv6_host_route(int index, const char *host,
 
 	memset(&rt, 0, sizeof(rt));
 
-	rt.rtmsg_dst_len = 128;
+	rt.rtmsg_dst_len = prefix_len;
 
 	err = inet_pton(AF_INET6, host, &rt.rtmsg_dst);
 	if (err < 0)
@@ -814,6 +822,12 @@ out:
 	return err;
 }
 
+int connman_inet_add_ipv6_host_route(int index, const char *host,
+					const char *gateway)
+{
+	return connman_inet_add_ipv6_network_route(index, host, gateway, 128);
+}
+
 int connman_inet_set_ipv6_gateway_address(int index, const char *gateway)
 {
 	struct in6_rtmsg rt;
